*** The Simple Option Case - One Option at a Time (Model 14.4)

sets
   i               'Set for the vector of future prices'           /1*20/
   j               'Number of call option prices available'        /1*26/
;

scalar r           'Risk free rate of return'      /0.00001/;
scalar T           'Days left until exercise'      /9/;
scalar S0          'Current price of the stock'    /51.35/;
scalar Kj          'Strike price at the j-th loop';
scalar Cj          'Call option price at the j-th loop';
parameter pp(i,j)  'Estimated risk-neutral probabilities';
parameter conv(j)  'Convergence';

parameter St(i)    'Future (likely) prices of the stock'    /
1         30.00
2         32.50
3         35.00
4         37.50
5         40.00
6         42.50
7         45.00
8         47.50
9         50.00
10        52.50
11        55.00
12        57.50
13        60.00
14        62.50
15        65.00
16        67.50
17        70.00
18        72.50
19        75.00
20        77.50
/;

parameter K(j)   'Strike prices' /
1         40.0
2         43.5
3         44.0
4         47.5
5         48.0
6         48.5
7         49.5
8         50.0
9         50.5
10        51.0
11        51.5
12        52.0
13        52.5
14        53.0
15        53.5
16        54.0
17        54.5
18        55.0
19        55.5
20        56.0
21        56.5
22        57.0
23        57.5
24        58.0
25        60.0
26        62.0
/;

parameter C(j)   'Call option prices'    /
1         11.60
2         8.09
3         7.60
4         3.15
5         2.69
6         2.50
7         1.55
8         1.40
9         1.19
10        1.07
11        0.67
12        0.46
13        0.29
14        0.15
15        0.12
16        0.06
17        0.05
18        0.03
19        0.04
20        0.05
21        0.04
22        0.05
23        0.31
24        0.19
25        0.04
26        0.05
/;

variables
   obj           'Value of the objective function'
;

positive variables
   p(i)          'Probability Density for future prices'
;

equations
   eq_obj        'Equation - Objective function'
   eq_moment     'Moment constraints as call option prices'
   eq_stock      'Expected price of stock'
   eq_add        'Additivity constraint for probabilities'
;

eq_obj..         obj=e=-sum(i,p(i)*log(p(i)+1.e-8));
eq_moment..      Cj=e=exp(-r*T)*sum(i,max(St(i)-Kj,0)*p(i));
eq_stock..       S0=e=exp(-r*T)*sum(i,St(i)*p(i));
eq_add..         sum(i,p(i))=e=1;

model OptionME   'Option Pricing model based Maximum Entropy' /
all
/;

* Initial values
p.l(i)=1/card(i);

*options nlp=pathnlp;
options nlp=conopt;
option RESLIM=1000;
option decimals=4;

Loop(j,
   Cj=C(j);
   Kj=K(j);
   solve OptionME maximizing obj using nlp;
   pp(i,j)=p.l(i);
   conv(j)=OptionME.Modelstat
);

display pp, conv;

*execute_unload "OptionME.gdx" pp;
*execute 'gdxxrw.exe OptionME.gdx par=pp'
